define(["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    let jsyaml = window.jsyaml;
    exports.jsyaml = jsyaml;
});
