html {
  font-size: 14px;
}

body {
  line-height: 1.6;
  font-weight: 500;
  font-family: "Source Sans Pro", "HelveticaNeue", "Helvetica Neue", Helvetica, Arial, sans-serif;
  color: #dcdde1;
  margin: 0;
  padding: 0;
  background: #2f3640;
  text-rendering: optimizeLegibility;
  -webkit-font-smoothing: antialiased;
  -moz-osx-font-smoothing: grayscale;
}

.container {
  width: 1024px;
  padding-right: 15px;
  padding-left: 15px;
  margin-right: auto;
  margin-left: auto;
}

.experiments_list {
  margin-top: 15px;
}
.experiments_list .experiment {
  padding: 5px;
  cursor: pointer;
}
.experiments_list .experiment h3 {
  margin-top: 0;
  margin-bottom: 8px;
}
.experiments_list .experiment:hover {
  background: #353b48;
  color: #f5f6fa;
}

.experiment_single .run {
  cursor: pointer;
  padding: 5px;
  margin: 10px 0;
}
.experiment_single .run h3 {
  margin-top: 0;
  margin-bottom: 0;
}
.experiment_single .run h4 {
  margin-top: 0;
  margin-bottom: 8px;
}
.experiment_single .run h4 label {
  color: #718093;
}
.experiment_single .run:hover {
  background: #353b48;
}

.block {
  margin: 10px 0;
  box-shadow: 0 5px 10px -9px rgba(0, 0, 0, 0.7);
  border-radius: 10px;
  border: 1px solid rgba(255, 255, 255, 0.05);
}
.block > h3 {
  margin: 0;
  padding: 5px;
  font-weight: normal;
}

.run_single .key_value:hover {
  background: #353b48;
  color: #f5f6fa;
}
.run_single div.controls {
  margin-bottom: 12px;
}

.info_list > span {
  display: inline-block;
  overflow: hidden;
}
.info_list > span.key {
  margin-right: 5px;
  width: 200px;
  opacity: 0.7;
  white-space: nowrap;
  text-overflow: ellipsis;
}
.info_list > span.key + span:before {
  content: ":";
  opacity: 0.3 !important;
  margin-right: 5px;
  font-family: "Source Sans Pro", "HelveticaNeue", "Helvetica Neue", Helvetica, Arial, sans-serif;
}
.info_list.mono span.value {
  font-family: monospace;
}
.info_list.highlight {
  font-weight: bold;
}

.configs .info_list.only_option > span.key {
  opacity: 0.4;
}
.configs .info_list.only_option > span {
  opacity: 0.4;
}
.configs .info_list.common > span.key {
  opacity: 0.4;
}
.configs .info_list.common > span {
  opacity: 0.4;
}
.configs .info_list.ignored > span.key {
  opacity: 0.4;
}
.configs .info_list.ignored > span {
  opacity: 0.4;
  text-decoration: line-through;
}

.config .computed {
  margin-right: 5px;
  font-family: monospace;
}
.config > .option {
  margin-right: 5px;
  font-family: monospace;
  color: #9c88ff;
}
.config .options {
  font-family: monospace;
}
.config.only_option > .option {
  opacity: 0.8;
}

.collapsed .info_list.collapsible {
  display: none;
}

.block .info_list {
  padding: 0 10px;
  line-height: 0;
}
.block .info_list > span {
  line-height: 2;
}
.block .info_list:hover {
  background: #555b68 !important;
}
.block .info_list:hover > span {
  opacity: 1 !important;
  text-decoration: none;
}
.block .info_list:nth-child(even) {
  background: #353b48;
}

.collapsed .info_list:nth-child(even) {
  background: none;
}
.collapsed .not_collapsible:nth-child(even) {
  background: #353b48;
}

.key_icon {
  margin-right: 8px;
  opacity: 0.3;
}

.key_split {
  display: inline-block;
  width: 16px;
}

button {
  font-family: "Source Sans Pro", "HelveticaNeue", "Helvetica Neue", Helvetica, Arial, sans-serif;
  display: inline-block;
  color: #00a8ff;
  text-align: center;
  vertical-align: middle;
  cursor: pointer;
  user-select: none;
  border: none;
  background: transparent;
  font-size: 1rem;
  padding: 0.375rem 0.75rem;
  margin-right: 12px;
}
button:hover {
  opacity: 0.8;
}
button:active {
  opacity: 0.5;
}
button::-moz-focus-inner {
  border: 0;
}
button.small {
  font-size: 0.8rem;
  padding: 0;
  border: none;
}
button.inline {
  padding: 0;
  border: none;
  vertical-align: baseline;
}
button.danger {
  color: #e84118;
  border-color: #e84118;
}
button:disabled {
  cursor: not-allowed;
  color: #656670;
  border-color: #656670;
}

.boolean {
  color: #fbc531;
}

.int {
  color: #4cd137;
}

.float {
  color: #4cd137;
}

.string {
  color: #e84118;
}

.unknown {
  color: #f5f6fa;
}

.subtle {
  color: #ccc;
  font-size: 0.6rem;
}

.size_unit {
  color: #f5f6fa;
  font-family: "Source Sans Pro", "HelveticaNeue", "Helvetica Neue", Helvetica, Arial, sans-serif;
}

.size {
  opacity: 0.9;
}

.generation {
  font-size: 0.7em;
  color: #6ab04c;
  margin-right: 2px;
}

div.table {
  display: table;
  box-shadow: 0 0 5px -1px #000;
}
div.table div.row, div.table div.header {
  white-space: nowrap;
  line-height: 0;
}
div.table div.row {
  background: #2f3640;
}
div.table div.row:hover {
  background: #555b68;
}
div.table div.row.even {
  background: #353b48;
}
div.table div.row.even div.cell.config {
  background: #3f4247;
}
div.table div.row.even div.cell.value {
  background: #364347;
}
div.table div.row.even:hover div.cell {
  background: #555b68 !important;
}
div.table div.row.odd {
  background: #2f3640;
}
div.table div.row.odd div.cell.config {
  background: #393d3f;
}
div.table div.row.odd div.cell.value {
  background: #303e40;
}
div.table div.row.odd:hover div.cell {
  background: #555b68 !important;
}
div.table div.row.selected {
  background: #151b18;
}
div.table div.row.selected div.cell.config {
  background: #212419;
}
div.table div.row.selected div.cell.value {
  background: #18241a;
}
div.table div.row.selected:hover div.cell {
  background: #555b68 !important;
}
div.table div.header {
  background: #2f3640;
  box-shadow: 0px 5px 5px -5px #000;
  position: relative;
}
div.table div.header div.cell {
  font-weight: bold;
  line-height: 3;
  cursor: pointer;
}
div.table div.cell {
  display: inline-block;
  white-space: nowrap;
  overflow: hidden;
  text-overflow: ellipsis;
  border-right: 1px solid #555;
  padding: 0 4px;
  min-height: 1.5em;
  line-height: 2;
  user-select: none;
}
div.table div.cell.same {
  width: 1em !important;
  opacity: 0.3;
}
div.table div.cell.right {
  text-align: right;
}
div.table div.cell span.ignored {
  opacity: 0.25;
}
div.table div.cell span.controls i {
  margin-right: 8px;
  cursor: pointer;
}
div.table div.cell span.controls i.fa-file {
  color: #00a8ff;
}
div.table div.cell span.only_option {
  opacity: 0.6;
}
div.table div.cell span.picked {
  margin-right: 4px;
}
div.table div.cell span.custom {
  margin-right: 4px;
  color: #00a8ff;
}
div.control_panel {
  padding: 10px;
}
div.control_panel div.editor_controls i {
  margin: 8px;
  cursor: pointer;
}
div.control_panel div.editor {
  font-size: 0.9rem;
}
div.control_panel div.search {
  margin-top: 10px;
  margin-bottom: 4px;
}
div.control_panel div.search > input {
  width: 200px;
}
div.control_panel div.actions-container {
  margin-top: 10px;
}
div.control_panel div.actions-container div.actions {
  display: inline-block;
  vertical-align: baseline;
}
div.control_panel div.actions-container div.actions button {
  vertical-align: baseline;
}
div.control_panel div.actions-container div.selected_count {
  display: inline-block;
  vertical-align: baseline;
}
div.control_panel div.actions-container div.selected_count.items-selected {
  font-weight: bold;
}

div.input-container {
  background: #dcdde1;
  color: #2f3640;
  display: inline-block;
  vertical-align: baseline;
  line-height: 1rem;
  font-size: 1rem;
}
div.input-container .input-icon {
  padding: 5px;
}
div.input-container input {
  background: #dcdde1;
  color: #2f3640;
  outline: none;
  border: none;
  font-size: 1rem;
  padding: 5px;
}

.config .options::before, div.table div.cell span.options::before {
  content: "[";
  opacity: 0.7;
  color: #e84118;
}
.config .options::after, div.table div.cell span.options::after {
  content: "]";
  opacity: 0.7;
  color: #e84118;
}
.config .options span::after, div.table div.cell span.options span::after {
  content: ", ";
  opacity: 0.7;
  color: #e84118;
}
.config .options span:last-child::after, div.table div.cell span.options span:last-child::after {
  content: "";
  opacity: 0.7;
  color: #e84118;
}

div.table_container {
  display: table;
  position: relative;
}
div.table_container div.header_controls_background {
  position: fixed;
  top: 0;
  left: 0;
  right: 0;
  bottom: 0;
  background: #000;
  opacity: 0.4;
  z-index: 100;
  display: none;
}
div.table_container div.header_controls {
  display: none;
  position: absolute;
  z-index: 1000;
  background: #2f3640;
}
div.table_container div.header_controls i {
  padding: 4px;
  margin: 4px;
}
div.table_container div.header_controls i:hover {
  background: #353b48;
  color: #f5f6fa;
  cursor: pointer;
}

/*# sourceMappingURL=style.css.map */
